part of tecfy_core_client_package;

class PropertyDateTime extends Property {
  DateTime? minDate;
  DateTime? maxDate;

  bool isTodayByDefault = false;

  bool isAllowFuture = true;
  bool isAllowPast = true;

  /// <summary>
  /// yyyy-MM-dd    (default)
  /// dd/MM/yyyy
  /// </summary>
  String format = "yyyy-MM-dd";

  PropertyDateTime(
      {required String dbName,
      required String nameAr,
      required String nameEn,
      bool isRequired = false,
      this.format = 'yyyy-MM-dd',
      this.isTodayByDefault = false})
      : super(
            dbName: dbName,
            nameAr: nameAr,
            nameEn: nameEn,
            propertyType: PropertyTypes.Date,
            valueType: ValueTypes.DateTime,
            isRequired: isRequired);

  PropertyDateTime.fromMap(Map<String, dynamic> row) : super.fromMap(row) {
    minDate = getDateFromJsonString(row['minDate'] ?? '2000-01-01');
    maxDate = getDateFromJsonString(row['maxDate'] ?? '2050-01-01');
    isTodayByDefault = row['isTodayByDefault'] ?? false;
    isAllowFuture = row['isAllowFuture'] ?? true;
    isAllowPast = row['isAllowPast'] ?? true;
    format = row['format'] ?? 'yyyy-MM-dd';
  }

  DateTime? getDateFromJsonString(String value) {
    return BaseModel.parseUTC(value);
    // return DateTime.parse(value,);
  }

  @override
  OperationResult isValid(data) {
    if (!super.isValid(data).success) return super.isValid(data);

    return OperationResult(success: true);
  }

  Widget getDisplayWidget(Map<String, dynamic> data,
      {double size = 12, Color color = Colors.black}) {
    if (data[dbName] == null) {
      return Text('');
    }
    if (valueType == ValueTypes.Number) {
      return Text(
          data[dbName] == null || data[dbName] == 0
              ? ''
              : intl.DateFormat(format).format(
                  DateTime.fromMillisecondsSinceEpoch(data[dbName],
                      isUtc: true)),
          style: TextStyle(fontSize: size, color: color));
    } else {
      return Text(data[dbName].toString().replaceAll('T', ' '),
          style: TextStyle(fontSize: size, color: color));
    }
  }

  Future<bool> getDefaultValue(Map<String, dynamic> data) async {
    if (data[dbName] != null) return false;
    if (isTodayByDefault) {
      data[dbName] = DateTime.now().toUtc();
      return true;
    }
    return false;
  }
}
