part of tecfy_core_client_package;

class LoginPage extends StatefulWidget {
  @override
  _LoginPageState createState() => _LoginPageState();
}

class _LoginPageState extends AppStateView<LoginPage, LoginController> {
  @override
  void initState() {
    _controller = LoginController(this);
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        backgroundColor: ThemeService.mainColor,
        //resizeToAvoidBottomPadding :false,
        body: LayoutService.bindWidget(context,
            mobileWidget: getMobileWidget(),
            descktopWidget: getDesctopWidget()));
  }

  Widget getDesctopWidget() {
    return Row(
      children: <Widget>[
        Container(
            width: 350,
            decoration: TecfyCoreApp.backgoundImage == null
                ? null
                : BoxDecoration(
                    image: DecorationImage(
                        image: TecfyCoreApp.backgoundImage.image,
                        fit: BoxFit.cover),
                  ),
            child: Column(
                mainAxisAlignment: MainAxisAlignment.end,
                children: <Widget>[
                  SizedBox(height: 5),
                  TecfyPoweredBy(),
                  SizedBox(height: 5),
                  Text(
                    'Version ${TecfyCoreApp.packageInfo.version} © 2021',
                    style: TextStyle(
                        fontSize: 12,
                        color: ThemeService.secondryColor,
                        fontWeight: FontWeight.normal),
                  ),
                  SizedBox(height: 5),
                ])
            //child: TecfyCoreApp.logo == null ? Container(): Center(child: Image(image: TecfyCoreApp.logo.image, height :150.0, width: 200.0))
            ),
        Expanded(
            child: Container(
                decoration: new BoxDecoration(boxShadow: [
                  BoxShadow(
                    color: ThemeService.mainColor,
                    blurRadius: 20.0, // has the effect of softening the shadow
                    spreadRadius: 5.0, // has the effect of extending the shadow
                    offset: Offset(
                      5.0, // horizontal, move right 10
                      5.0, // vertical, move down 10
                    ),
                  )
                ]),
                child: Center(
                    child: Container(
                        constraints:
                            BoxConstraints(minWidth: 100, maxWidth: 500),
                        padding: EdgeInsets.symmetric(horizontal: 30.0),
                        child: getLoginForm()))))
      ],
    );
  }

  Widget getMobileWidget() {
    return SizedBox(
        width: LayoutService.getWidth(context),
        child: Column(
          children: <Widget>[
            Container(
              height:
                  MediaQuery.of(context).viewInsets.bottom == 0 ? 250.0 : 120.0,
              //color: Colors.white,
              // child: TecfyCoreApp.logo == null
              //     ? Container()
              //     : Center(
              //         child: Image(
              //             image: TecfyCoreApp.logo.image,
              //             height: 150.0,
              //             width: 200.0),
              //       ),
              decoration: TecfyCoreApp.backgoundImage == null
                  ? null
                  : BoxDecoration(
                      image: DecorationImage(
                          image: TecfyCoreApp.backgoundImage.image,
                          fit: BoxFit.cover),
                    ),
            ),
            Expanded(
                child: Container(
                    constraints: BoxConstraints(minWidth: 100, maxWidth: 500),
                    padding: EdgeInsets.symmetric(horizontal: 30.0),
                    child: getLoginForm()))
          ],
        ));
  }

  Widget getLoginForm() {
    return Form(
      key: _controller.formKey,
      child: SingleChildScrollView(
          child: Column(
        children: <Widget>[
          SizedBox(height: 50.0),
          AppTextFormField(
              labelText: AppLocale.translate(context, 'Email or Username'),
              controller: _controller.usernameController,
              validator: _controller.usernameValidator,
              icon: Icon(Icons.person),
              isDarkBackgound: true,
              keyboardType: TextInputType.emailAddress),
          AppTextFormField(
              labelText: AppLocale.translate(context, 'Password'),
              controller: _controller.passwordController,
              validator: _controller.passwordValidator,
              icon: Icon(Icons.vpn_key),
              isDarkBackgound: true,
              obscureText: true),
          SizedBox(height: 50.0),
          // TextButton(child: Text('LOG IN2'), onPressed: _controller.onLogin),
          AppButtonPost(
              label: AppLocale.translate(context, 'LOG IN'),
              onPressed: _controller.onLogin),
          SizedBox(height: 30.0),
          Row(
            children: <Widget>[
              Expanded(child: Divider(color: ThemeService.textOnMainColor)),
              Padding(
                  padding: EdgeInsets.symmetric(horizontal: 10.0),
                  child: Text(AppLocale.translate(context, 'OR'),
                      style: TextStyle(
                          fontSize: 11.0,
                          color: ThemeService.textOnMainColor))),
              Expanded(child: Divider(color: ThemeService.textOnMainColor)),
            ],
          ),
          //SizedBox(height: 5.0),
          TextButton(
              child: Text(AppLocale.translate(context, 'Contact Us'),
                  style: TextStyle(color: ThemeService.subColor)),
              onPressed: _controller.contactUs),
          SizedBox(height: 10.0),
          Row(
            mainAxisAlignment: MainAxisAlignment.center,
            crossAxisAlignment: CrossAxisAlignment.center,
            children: <Widget>[
              ConnectionService.getConnectionIconStream(),
              IconButton(
                  icon: Icon(Icons.language),
                  color: ThemeService.textOnMainColor,
                  tooltip: LocalConfigService.language == Language.arabic
                      ? 'English'
                      : 'عربي',
                  onPressed: () {
                    LocalConfigService.language =
                        LocalConfigService.language == Language.arabic
                            ? Language.english
                            : Language.arabic;
                  })
            ],
          )
        ],
      )),
    );
  }
}
