part of tecfy_core_client_package;

class BarcodePropertyComponenet extends StatelessWidget {
  final TextInputAction? textInputAction;
  final TextEditingController? controller;
  final String Function(String?)? validator;
  final void Function()? onEditingComplete;
  final void Function(String)? onFieldSubmitted;
  final TextInputType? keyboardType;
  final String? labelText;
  final bool isRequired;
  final bool obscureText;
  final bool autoFocus;
  final FocusNode? focusNode;
  final bool isDarkBackgound;
  final TextDirection? textDirection;
  final TextStyle? textStyle;

  BarcodePropertyComponenet(
      {this.textInputAction,
      this.controller,
      this.isDarkBackgound = false,
      //this.icon = ,
      this.textDirection,
      this.validator,
      this.keyboardType,
      this.labelText,
      this.isRequired = false,
      this.onEditingComplete,
      this.focusNode,
      this.obscureText = false,
      this.autoFocus = false,
      this.onFieldSubmitted,
      this.textStyle});

  TextStyle get defaultTextStyle => TextStyle(
      color: isDarkBackgound ? Colors.white : ThemeService.mainTextColor);

  @override
  Widget build(BuildContext context) {
    Widget result;
    result = !PlatformService.isBarcodeSupported
        ? getSingleRowText(context)
        : Stack(children: <Widget>[
            getSingleRowText(context),
            Container(
                height: 60,
                padding: EdgeInsets.only(right: 10, left: 10),
                child: Align(
                    child: TextButton.icon(
                      onPressed: () => scanQR(context),
                      icon: Icon(Icons.qr_code),
                      label: Text(""),
                    ),
                    alignment: ConfigService.isRTL
                        ? Alignment.centerLeft
                        : Alignment.centerRight))
          ]);

    return isDarkBackgound
        ? Theme(child: result, data: ThemeData.dark())
        : result;
  }

  Widget getSingleRowText(context) {
    return Theme.of(context).platform == TargetPlatform.iOS
        ? Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: <Widget>[
                Text(
                  '$labelText ${isRequired ? '*' : ''}',
                  style: TextStyle(
                      fontSize: 12.0,
                      color: isDarkBackgound ? Colors.white : Colors.black38),
                ),
                CupertinoTextField(
                    controller: controller,
                    textInputAction: textInputAction,
                    keyboardType: keyboardType,
                    //validator: validator,
                    onEditingComplete: onEditingComplete,
                    obscureText: obscureText,
                    maxLines: 1,
                    placeholder: labelText,
                    focusNode: focusNode,
                    autofocus: autoFocus,
                    onSubmitted: onFieldSubmitted,
                    style: textStyle ?? defaultTextStyle,
                    decoration: BoxDecoration()),
                Container(
                    height: 1.0,
                    color: isDarkBackgound ? Colors.white : Colors.black),
                SizedBox(height: 15.0)
              ])
        : TextFormField(
            textDirection: textDirection,
            enableInteractiveSelection: !obscureText,
            textInputAction: textInputAction,
            keyboardType: keyboardType,
            controller: controller,
            onEditingComplete: onEditingComplete,
            focusNode: focusNode,
            validator: validator,
            obscureText: obscureText,
            maxLines: 1,
            autofocus: autoFocus,
            style: textStyle ?? defaultTextStyle,
            onFieldSubmitted: onFieldSubmitted,
            enableSuggestions: true,
            decoration: labelText == null
                ? null
                : InputDecoration(
                    labelText: isRequired ? '$labelText*' : labelText,
                    border: UnderlineInputBorder(),
                  ),
          );
  }

  void scanQR(context) async {
    var result = await AppNavigator.showAppDialog(
        context, (c) => ScannerCameraDialog(labelText ?? 'Barcode'));

    if (result != null) {
      controller?.text = result;
      onEditingComplete?.call();
    }
  }
}
