part of tecfy_core_client_package;

class DropdownAdvancedEntity extends StatefulWidget {
  final Entity entity;
  final String entityId;
  final String? url;
  final String? label;
  final bool isRequired;
  final Map<String, dynamic> data;
  final PropertyDropdown property;
  final void Function(String, Map<String, dynamic>?)? onChanged;
  DropdownAdvancedEntity(this.entityId,
      {required this.property,
      this.label,
      this.isRequired = false,
      required this.entity,
      this.url,
      this.onChanged,
      required this.data});
  @override
  _DropdownAdvancedEntityState createState() => _DropdownAdvancedEntityState();
}

class _DropdownAdvancedEntityState extends AppState<DropdownAdvancedEntity> {
  List<dynamic>? listData;

  String get _value => dataValue?['id'];
  String? _initValue;
  Map<String, dynamic>? dataValue;
  Entity? _entity;
  String? entityPropName;
  dynamic defaultParentData = {'id': null};

  @override
  void initState() {
    super.initState();
    init();
  }

  init() {
    _initValue = widget.data[widget.property.dbName];

    if (_entity == null) _entity = widget.entity;
    if (entityPropName == null)
      entityPropName = widget.property.dbName
          .substring(0, widget.property.dbName.indexOf("Id"));

    if (dataValue == null || dataValue!['id'] != _initValue) {
      if (widget.data[entityPropName] != null &&
          widget.data[entityPropName]['id'] == _initValue)
        dataValue = widget.data[entityPropName];
    }

    if (dataValue == null && _initValue != null) {
      // load data from server
      EntityService.getOne(widget.entity.dbName, _initValue).then((result) {
        if (result != null) {
          dataValue = result;
          widget.data[entityPropName!] = result;
          if (mounted) setState(() {});
        }
      });
    }
  }

  _onItemSelectedFromList(context, v) {
    try {
      dataValue = v;
      widget.data[entityPropName!] = dataValue;
      if (widget.onChanged != null) widget.onChanged?.call(_value, dataValue);
      setState(() {});
      context != null ? Navigator.pop(context) : print("null Navigator");
    } catch (err, trac) {
      print(err);
      print(trac);
    }
  }

  _onSaveNewItem(BuildContext context, newDataRaw) async {
    try {
      var validateResult = EntityService.validateEntity(_entity, newDataRaw);
      if (!validateResult.success) {
        AppNavigator.showMessage(context, validateResult.message ?? 'Error');
        return;
      }
      var addResult =
          await EntityService.saveEntity(_entity, newDataRaw, context, true);
      if (addResult.success) {
        dataValue = addResult.data;
        widget.data[entityPropName!] = dataValue;
        _initValue = dataValue!['id'];
        widget.onChanged?.call(_initValue!, dataValue!);
        Navigator.pop(context, true); // close the add new item's form
      } else {
        AppNavigator.showMessage(context, addResult.message ?? 'Error');
      }
    } catch (err, trace) {
      print('XXXXXXXXX erorr adding item');
      print(err);
      print(trace);
      AppNavigator.showMessage(context, 'Error Adding Item');
    }
  }

  openSearchDialog() async {
    await AppNavigator.showAppDialog(
        context,
        (listDialogContext) => Column(
              mainAxisAlignment: MainAxisAlignment.center,
              crossAxisAlignment: CrossAxisAlignment.center,
              children: <Widget>[
                Expanded(child: Container()),
                Container(
                    width: min(500, MediaQuery.of(context).size.width),
                    height: min(
                        800,
                        MediaQuery.of(context).size.height -
                            MediaQuery.of(context).padding.top -
                            MediaQuery.of(context).padding.bottom -
                            50),
                    child: Card(
                        child: Column(
                            mainAxisAlignment: MainAxisAlignment.start,
                            crossAxisAlignment: CrossAxisAlignment.stretch,
                            children: <Widget>[
                          Container(
                              color: ThemeService.backgroundSecondColor,
                              child: Row(
                                  mainAxisAlignment:
                                      MainAxisAlignment.spaceBetween,
                                  children: <Widget>[
                                    Padding(
                                        padding: EdgeInsets.only(left: 10),
                                        child: Text(
                                            '${widget.label}${widget.isRequired ? '*' : ''}',
                                            style: ThemeService.titleStyle)),
                                    IconButton(
                                        icon: Icon(Icons.close),
                                        onPressed: () =>
                                            Navigator.pop(listDialogContext))
                                  ])),
                          Expanded(
                              child: EntityListSearchComponent(
                            property: widget.property,
                            entity: widget.entity,
                            url: widget.url ?? '',
                            data: widget.data,
                            // isFilter: widget.isFilter ,
                            // filterValue: widget.filterValue,
                            // filterField: widget.filterField,
                            urlFilterData: widget.data,
                            onClick: (v) =>
                                _onItemSelectedFromList(listDialogContext, v),
                          )),
                          TextButton(
                            child: Text(trans('Add New Item'),
                                style: ThemeService.flatButtonTextStyle
                                    .copyWith(
                                        decoration: TextDecoration.underline)),
                            onPressed: () async {
                              Map<String, dynamic> newDataRaw =
                                  new Map<String, dynamic>();
                              var addNewResult =
                                  await AppNavigator.showAppDialog(
                                      listDialogContext,
                                      (addDialogContext) => _getAddNewDialog(
                                          addDialogContext, newDataRaw));
                              if (addNewResult ?? false)
                                Navigator.pop(
                                    listDialogContext); // close the search form
                            },
                          )
                        ]))),
                Expanded(child: Container()),
              ],
            ));
    //loadData();
  }

  Widget _getAddNewDialog(addDialogContext, newDataRaw) {
    return Container(
      constraints: BoxConstraints(maxHeight: 500, maxWidth: 500),
      child: Center(
        child: Card(
          child: ListView(
            children: <Widget>[
              InLineForm(widget.entityId,
                  entity: _entity!,
                  inPopup: true,
                  data: newDataRaw,
                  onChanged: (v) {}),
              Padding(
                  padding: EdgeInsets.symmetric(horizontal: 20),
                  child: Row(
                    children: <Widget>[
                      Expanded(child: Container()),
                      Expanded(
                          child: AppButtonPost(
                              label: trans('Save'),
                              onPressed: () => _onSaveNewItem(
                                  addDialogContext, newDataRaw))),
                      SizedBox(width: 10),
                      Expanded(
                          child: Container(
                              height: 50.0,
                              child: TextButton(
                                  child: Text(trans('Cancel'),
                                      style: ThemeService.flatButtonTextStyle),
                                  onPressed: () =>
                                      Navigator.pop(addDialogContext, false)))),
                      Expanded(child: Container()),
                    ],
                  )),
            ],
          ),
        ),
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    if (_initValue != widget.data[widget.property.dbName] ||
        dataValue == null ||
        dataValue!['id'] != _initValue) init();
    print('----------- ${_entity!.displayField.dbName}');
    if (dataValue != null && _entity!.displayField.dbName != null)
      print("${dataValue![_entity!.displayField.dbName]}");
    return dataValue != null
        ? Stack(children: [
            MaterialButton(
                padding: EdgeInsets.all(0),
                onPressed: openSearchDialog,
                child: widget.label == null
                    ? Text(
                        dataValue == null
                            ? trans('Click To Select')
                            : (dataValue![_entity!.displayField.dbName] ??
                                'N/A'),
                        style: TextStyle(color: Colors.black, fontSize: 15))
                    : Column(
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: <Widget>[
                          SizedBox(height: 10),
                          Text(
                              '${widget.label ?? ''}${widget.isRequired ? '*' : ''}',
                              style: TextStyle(
                                  color: ThemeService.textSubColor,
                                  fontSize: 11)),
                          Text(
                              dataValue == null
                                  ? trans('Click To Select')
                                  : (dataValue![_entity!.displayField.dbName]
                                          ?.toString() ??
                                      'N/A'),
                              style:
                                  TextStyle(color: Colors.black, fontSize: 15)),
                          SizedBox(height: 10),
                          Container(
                              height: 1, color: ThemeService.textSubColor),
                          SizedBox(height: 5),
                        ],
                      )),
            Align(
              alignment: Alignment.topLeft,
              child: IconButton(
                  onPressed: () {
                    _onItemSelectedFromList(null, null);
                  },
                  icon: Icon(Icons.close, color: Colors.grey),
                  tooltip: trans("Clean")),
            )
          ])
        : MaterialButton(
            padding: EdgeInsets.all(0),
            onPressed: openSearchDialog,
            child: widget.label == null
                ? Text(
                    dataValue == null
                        ? trans('Click To Select')
                        : (dataValue![_entity!.displayField.dbName] ?? 'N/A'),
                    style: TextStyle(color: Colors.black, fontSize: 15))
                : Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: <Widget>[
                      SizedBox(height: 10),
                      Text(
                          '${widget.label ?? ''}${widget.isRequired ? '*' : ''}',
                          style: TextStyle(
                              color: ThemeService.textSubColor, fontSize: 11)),
                      Text(
                          dataValue == null
                              ? trans('Click To Select')
                              : (dataValue![_entity!.displayField.dbName]
                                      ?.toString() ??
                                  'N/A'),
                          style: TextStyle(color: Colors.black, fontSize: 15)),
                      SizedBox(height: 10),
                      Container(height: 1, color: ThemeService.textSubColor),
                      SizedBox(height: 5),
                    ],
                  ));
  }
}
