part of tecfy_core_client_package;

class PorpertyComponent extends StatefulWidget {
  final Property property;
  final Entity? entity;
  final Color? valueColor;
  final bool noLabel;
  final void Function(dynamic)? onChange;
  final void Function(dynamic)? onSubmit;
  final void Function(Map<String, dynamic>)? onUpdateData;
  //final dynamic initalValue;
  /// if this property is a dropdown and it's filterd by another field
  //final dynamic filterValue;
  final Map<String, dynamic> data;
  final bool readOnly;
  PorpertyComponent(
      {required this.property,
      this.entity,
      this.onChange,
      this.onSubmit,
      this.onUpdateData,
      this.valueColor,
      this.noLabel = false,
      //this.filterValue,
      required this.data,
      this.readOnly =
          false}); //: assert(entity != null || property.isRequired == false);
  @override
  _PorpertyComponentState createState() => _PorpertyComponentState();
}

class _PorpertyComponentState extends State<PorpertyComponent> {
  TextEditingController? textController;
  FocusNode? focusNode;
  dynamic _initValue;
  //bool isVisable =true;
  bool _isVisible = true;
  @override
  void initState() {
    init();
    super.initState();
  }

  init() {
    _initValue = widget.data[widget.property.dbName];
    var val = _initValue?.toString();
    if (widget.property.propertyType == PropertyTypes.Number && val != null) {
      if (val.contains('.'))
        val = (_initValue as num).toDouble().toStringAsFixed(2);
    }
    if (widget.property.propertyType == PropertyTypes.Text ||
        widget.property.propertyType == PropertyTypes.Number) {
      if (textController == null) {
        textController = TextEditingController(text: val ?? '');
        focusNode = FocusNode();
        textController!.addListener(() {
          submitChages();
        });
      } else {
        textController?.text = val ?? '';
      }
    }
  }

  submitChages() {
    if (widget.property.propertyType == PropertyTypes.Text) {
      _initValue = textController?.text;
      if (widget.onChange != null) widget.onChange?.call(textController?.text);
    }
    if (widget.property.propertyType == PropertyTypes.Number) {
      _initValue = num.tryParse(textController?.text ?? '');
      if (widget.onChange != null) widget.onChange?.call(_initValue);
    }
  }

  getInitValueString() {
    return _initValue == null
        ? '__'
        : _initValue is int
            ? _initValue.toString()
            : _initValue is num
                ? (_initValue as num).toDouble().toStringAsFixed(2)
                : _initValue.toString();
  }

  @override
  Widget build(BuildContext context) {
    widget.property.isVisible(widget.data, widget.entity).then((isVisible) {
      if (isVisible != _isVisible) {
        _isVisible = isVisible;
        setState(() {});
      }
    });

    if (!_isVisible) return Container();
    //if (!widget.property.isVisable) return Container();

    if (_initValue != widget.data[widget.property.dbName]) init();

    if (widget.readOnly) return getReadOnlyWidget();

    if (widget.property.readOnly &&
        widget.property.propertyType != PropertyTypes.Dropdown &&
        widget.property.propertyType != PropertyTypes.OtherEntityMany &&
        widget.property.propertyType != PropertyTypes.ManyToMany &&
        widget.property.propertyType != PropertyTypes.OneToMany) {
      if (widget.noLabel)
        return Text(getInitValueString());
      else
        return TextFormLabel(
          label: widget.property.name,
          value: getInitValueString(),
        );
    }
    switch (widget.property.propertyType) {
      case PropertyTypes.Dropdown:
        return DropdownAdvancedEntity(
            (widget.property as PropertyDropdown).entityName,
            label: widget.noLabel ? null : widget.property.name,
            isRequired: widget.noLabel ? false : widget.property.isRequired,
            entity: EntityService.getEntity(
                (widget.property as PropertyDropdown).entityName)!,
            url: (widget.property as PropertyDropdown).dropdownUrl,
            property: (widget.property as PropertyDropdown),
            data: widget.data, onChanged: (v, data) {
          widget.onChange?.call(v);
          if (data != null && widget.onUpdateData != null)
            widget.onUpdateData?.call(data);
        });
      case PropertyTypes.OtherEntityMany:
        return ListFormProperty(
            label: widget.noLabel ? null : widget.property.name,
            isRequired: widget.noLabel ? false : widget.property.isRequired,
            entityId: (widget.property as PropertyOtherEntityMany).entityName,
            data: _initValue ?? [],
            onChanged: (v) {
              widget.onChange?.call(v);
            });

      case PropertyTypes.ManyToMany:
        if (!(widget.property as PropertyManyToMany).isWithData) {
          return ManyToManyProperty(
            label: widget.noLabel ? null : widget.property.name,
            isRequired: widget.noLabel ? false : widget.property.isRequired,
            selectedValues: _initValue,
            entity: EntityService.getEntity(
                (widget.property as PropertyManyToMany).entityName)!,
            onChanged: (v) => widget.onChange?.call(v),
          );
        }

        return ManyToManyWithData(
          (widget.property as PropertyManyToMany).entityName,
          label: widget.noLabel ? null : widget.property.name,
          isRequired: widget.noLabel ? false : widget.property.isRequired,
          items: _initValue,
          entity: EntityService.getEntity(
              (widget.property as PropertyManyToMany).entityName)!,
          onChanged: (v) => widget.onChange?.call(v),
          entityMiddle:
              (widget.property as PropertyManyToMany).middleEntityName,
          propertieMiddle:
              (widget.property as PropertyManyToMany).middleOtherPropertyName,
        );

      case PropertyTypes.OneToMany:
        return ManyToManyProperty(
          label: widget.noLabel ? null : widget.property.name,
          isRequired: widget.noLabel ? false : widget.property.isRequired,
          selectedValues: _initValue,
          entity: EntityService.getEntity(
              (widget.property as PropertyOneToMany).entityName)!,
          onChanged: (v) => widget.onChange?.call(v),
        );

      case PropertyTypes.Text:
        if ((widget.property as PropertyText).isBarcode) {
          return BarcodePropertyComponenet(
            labelText: widget.noLabel ? null : widget.property.name,
            isRequired: widget.noLabel ? false : widget.property.isRequired,
            textStyle: widget.valueColor == null
                ? null
                : TextStyle(color: widget.valueColor),
            focusNode: focusNode,
            controller: textController,
            onFieldSubmitted: (v) {
              focusNode?.unfocus();
              widget.onSubmit?.call(_initValue);
            },
            onEditingComplete: submitChages,
          );
        }
        return AppTextFormField(
            textDirection: LocalConfigService.isArabic &&
                    !(widget.property as PropertyText).isEmail
                ? TextDirection.rtl
                : TextDirection.ltr,
            labelText: widget.noLabel ? null : widget.property.name,
            isRequired: widget.noLabel ? false : widget.property.isRequired,
            textStyle: widget.valueColor == null
                ? null
                : TextStyle(color: widget.valueColor),
            focusNode: focusNode,
            controller: textController,
            onFieldSubmitted: (v) {
              focusNode?.unfocus();
              widget.onSubmit?.call(_initValue);
            },
            textInputAction: TextInputAction.next,
            obscureText: (widget.property as PropertyText).isPassword,
            keyboardType: (widget.property as PropertyText).isEmail
                ? TextInputType.emailAddress
                : (widget.property as PropertyText).lines > 1
                    ? TextInputType.multiline
                    : TextInputType.text,
            maxLines: (widget.property as PropertyText).lines,
            onEditingComplete: submitChages);
      case PropertyTypes.Number:
        return AppTextFormField(
            labelText: widget.noLabel ? null : widget.property.name,
            isRequired: widget.noLabel ? false : widget.property.isRequired,
            controller: textController,
            focusNode: focusNode,
            onFieldSubmitted: (v) => focusNode?.unfocus(),
            textStyle: widget.valueColor == null
                ? null
                : TextStyle(color: widget.valueColor),
            textInputAction: TextInputAction.next,
            keyboardType: TextInputType.number,
            onEditingComplete: submitChages);
      case PropertyTypes.Boolean:
        return AppCheckBox(
            label: widget.noLabel ? null : widget.property.name,
            initalValue: _initValue,
            isFullrow: true,
            onChanged: (v) {
              widget.onChange?.call(v);
            });
      case PropertyTypes.File:
        return PhotoProperty(
            fileData: _initValue,
            label: widget.noLabel ? null : widget.property.name,
            isRequired: widget.noLabel ? false : widget.property.isRequired,
            fileId: widget.data[widget.property.dbName + 'Id'],
            onChanged: (v) => widget.onChange?.call(v));
      case PropertyTypes.Color:
        return ColorPropertyComponenet(
            initalValue: _initValue?.toString(),
            label: widget.noLabel ? null : widget.property.name,
            isRequired: widget.noLabel ? false : widget.property.isRequired,
            onChanged: (v) => widget.onChange?.call(v));
      case PropertyTypes.MapArea:
        return MapAreaPropertyComponenet(
            initalValue: _initValue,
            label: widget.noLabel ? null : widget.property.name,
            isRequired: widget.noLabel ? false : widget.property.isRequired,
            onChanged: (v) => widget.onChange?.call(v));
      case PropertyTypes.MarkImage:
        return MarkImagePropertyComponenet(
            initalValue: _initValue,
            label: widget.noLabel ? null : widget.property.name,
            isRequired: widget.noLabel ? false : widget.property.isRequired,
            onChanged: (v) => widget.onChange?.call(v));
      case PropertyTypes.Date:
        return DateProperty(
            label: widget.noLabel ? null : widget.property.name,
            isRequired: widget.noLabel ? false : widget.property.isRequired,
            initalValue: _initValue,
            onChanged: (v) {
              widget.onChange?.call(v);
            });
      case PropertyTypes.Location:
        return LocationPropertyComponenet(
            label: widget.noLabel ? null : widget.property.name,
            isRequired: widget.noLabel ? false : widget.property.isRequired,
            initalValue: _initValue,
            onChanged: (v) {
              widget.onChange?.call(v);
            });
      case PropertyTypes.MapArea:
        return MapAreaPropertyComponenet(
            label: widget.noLabel ? null : widget.property.name,
            isRequired: widget.noLabel ? false : widget.property.isRequired,
            initalValue: _initValue,
            onChanged: (v) {
              widget.onChange?.call(v);
            });
      default:
        return Text(
            '${widget.property.dbName} ${widget.property.propertyType}');
    }
  }

  Widget getReadOnlyWidget() {
    switch (widget.property.propertyType) {
      case PropertyTypes.Dropdown:
        var property = (widget.property as PropertyDropdown);
        var entity = EntityService.getEntity(property.entityName);

        String val;
        if (widget.data[property.dbNameEntity] == null) {
          EntityService.getOne(
                  entity!.dbName, widget.data[widget.property.dbName])
              .then((result) {
            if (result != null) {
              setState(() {
                widget.data[property.dbNameEntity] = result;
              });
            }
          });
          val = '__';
        } else {
          val = widget.data[property.dbNameEntity]
                  [entity!.displayField.dbName] ??
              widget.data[property.dbNameEntity]['display'] ??
              widget.data[property.dbNameEntity]['name'] ??
              widget.data[property.dbNameEntity]['nameAr'] ??
              widget.data[property.dbNameEntity]['nameEn'] ??
              widget.data[property.dbNameEntity]['title'] ??
              '__';
        }
        if (widget.noLabel)
          return Text(val);
        else
          return TextFormLabel(
            label: widget.property.name,
            value: val,
          );
        break;
      case PropertyTypes.OtherEntityMany:
        return ListFormProperty(
            readOnly: true,
            label: widget.noLabel ? null : widget.property.name,
            entityId: (widget.property as PropertyOtherEntityMany).entityName,
            data: _initValue ?? [],
            onChanged: (v) {});
      default:
        if (widget.noLabel) return Text(getInitValueString());
        try {
          return TextFormWidget(
            label: widget.noLabel ? null : widget.property.name,
            value: widget.property.getDisplayWidget(widget.data),
          );
        } catch (err) {
          return TextFormLabel(
            label: widget.property.name,
            value: getInitValueString(),
          );
        }
    }
  }
}
