part of tecfy_core_client_package;

class ScannerCameraDialog extends StatefulWidget {
  final String label;
  ScannerCameraDialog([this.label = 'Barcode Scanner']);
  @override
  _ScannerCameraDialogState createState() => _ScannerCameraDialogState();
}

class _ScannerCameraDialogState extends AppState<ScannerCameraDialog> {
  bool _isGranted = false;
  late ScannerController _scannerController;
  bool flashOpned = false;
  @override
  void initState() {
    init();
    super.initState();
    SystemChrome.setPreferredOrientations([
      DeviceOrientation.portraitUp,
    ]);
  }

  Future<String> enterBarcode() async {
    String code = '';
    return (await AppNavigator.showAppDialog(
                context,
                (c) => SimpleDialog(title: Text('Enter Barcode'), children: [
                      TextField(onChanged: (v) => code = v),
                      TextButton(
                        child: Text('OK'),
                        onPressed: () {
                          AppNavigator.pop(c, code);
                        },
                      )
                    ])))
            ?.toString() ??
        '';
  }

  init() async {
    try {
      var isGrant = await _requestPermission();
      if (isGrant) {
        _init();
      } else {
        AppNavigator.showMessage(context, 'We cannot get access to camera!')
            .then((d) {
          enterBarcode().then((e) async {
            await Future.delayed(Duration(microseconds: 1));
            AppNavigator.pop(context, e);
          });
        });
      }
    } catch (err) {
      AppNavigator.showMessage(context, 'We cannot get access to camera!')
          .then((d) {
        enterBarcode().then((e) async {
          await Future.delayed(Duration(microseconds: 1));
          AppNavigator.pop(context, e);
        });
      });
    }
  }

  @override
  void dispose() {
    super.dispose();

    _scannerController.stopCameraPreview();
    _scannerController.stopCamera();
    SystemChrome.setPreferredOrientations([
      DeviceOrientation.landscapeRight,
      DeviceOrientation.landscapeLeft,
      DeviceOrientation.portraitUp,
      DeviceOrientation.portraitDown,
    ]);
  }

  void _resultCallback(String? val) {
    if (val != null && val.isNotEmpty) {
      AppNavigator.pop(context, val);
    }
  }

  void _init() {
    _scannerController = ScannerController(scannerResult: (result) {
      _resultCallback(result);
    }, scannerViewCreated: () {
      TargetPlatform platform = Theme.of(context).platform;
      if (TargetPlatform.iOS == platform) {
        Future.delayed(Duration(seconds: 2), () {
          _scannerController.startCamera();
          _scannerController.startCameraPreview();
        });
      } else {
        _scannerController.startCamera();
        _scannerController.startCameraPreview();
      }
    });
    setState(() {});
  }

  Future<bool> _requestPermission() async {
    if (await perm.Permission.camera.request().isGranted) {
      setState(() {
        _isGranted = true;
      });
    }
    return _isGranted;
  }

  void _onFlash() {
    if (flashOpned == false) {
      _scannerController.openFlash();
      setState(() {
        flashOpned = true;
      });
    } else {
      _scannerController.closeFlash();
      setState(() {
        flashOpned = false;
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    return Material(
      child: Column(
        children: [
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: [
              IconButton(
                  icon: Icon(Icons.flash_on_outlined),
                  iconSize: 30,
                  color: flashOpned ? Colors.yellow : Colors.blue,
                  onPressed: _onFlash),
              Text(widget.label),
              TextButton.icon(
                onPressed: () => AppNavigator.pop(context),
                icon: Icon(Icons.close),
                label: Text(""),
              )
            ],
          ),
          Expanded(
            child: _isGranted
                ? PlatformAiBarcodeScannerWidget(
                    platformScannerController: _scannerController,
                    unsupportedDescription:
                        'Scanning is not supported in this device',
                  )
                : Center(child: CircularProgressIndicator()),
          ),
        ],
      ),
    );
  }
}
