part of tecfy_core_client_package;

class EntityFormPage extends StatefulWidget {
  final String entity;
  final dynamic data;
  final bool isPopup;
  EntityFormPage(this.entity, [this.data, this.isPopup = false]);
  @override
  _EntityFormPageState createState() => _EntityFormPageState();
}

class _EntityFormPageState
    extends AppStateView<EntityFormPage, EntityFormController> {
  bool isSaved = false;
  bool isMobile = false;
  var blancData = {};
  @override
  void initState() {
    _controller = EntityFormController(this, widget.entity, widget.data);
    if (widget.data == null) {
      if (mapEquals(
              ConfigService.getValueMap(
                  "savedDataForm${_controller.entity.name}"),
              {}) ==
          false) {
        Future.delayed(Duration.zero, () async {
          var getTheData = await AppNavigator.confirmMessage(
              context, "أستعادة البيانات السابقة");
          if (getTheData) {
            _controller.data = ConfigService.getValueMap(
                "savedDataForm${_controller.entity.name}");
            blancData = ConfigService.getValueMap(
                "savedDataForm${_controller.entity.name}");
            _controller.setState();
          } else {
            blancData = jsonDecode(jsonEncode(_controller.data));
          }
        });
      }
      Future.delayed(Duration(microseconds: 300), () {
        blancData = jsonDecode(jsonEncode(_controller.data));
      });
    }
    super.initState();
  }

  @override
  void dispose() {
    super.dispose();
    if (widget.data == null) {
      if (isSaved == false) {
        bool equalityMap(var mapx, var mapy) {
          for (var k in mapx.keys) {
            if (mapx[k] == null || mapx[k].toString().length == 0) continue;
            if (mapy[k] != mapx[k]) {
              return false;
            }
          }
          return true;
        }

        if (!equalityMap(_controller.data, blancData)) {
          ConfigService.setValueMap("savedDataForm${_controller.entity.name}",
              _controller.data ?? {});
        }
      }
    }
  }

  @override
  Widget build(BuildContext context) {
    isMobile = widget.isPopup || LayoutService.isMobile(context);
    return DefaultTabController(
      length: _controller.tabs.keys.length,
      child: BasePage(
        context,
        _controller.entity.name,
        isMobile: isMobile,
        appBarActions: !_controller.canSave
            ? []
            : <Widget>[
                isMobile
                    ? IconButton(
                        icon: Icon(Icons.save, color: ThemeService.mainColor),
                        //color: ThemeService.secondryColor,
                        tooltip: trans('Save'),
                        onPressed: () async {
                          if (await _controller.onSave())
                            setState(() => isSaved = true);
                        })
                    : AppButtonIconText(
                        icon: Icon(Icons.save,
                            color: ThemeService.textOnMainColor),
                        color: ThemeService.secondryColor,
                        label: trans('Save'),
                        onPressed: () async {
                          if (await _controller.onSave())
                            setState(() => isSaved = true);
                        }),
                _controller.isAddNew
                    ? Container()
                    : isMobile
                        ? IconButton(
                            icon: Icon(Icons.add_task,
                                color: ThemeService.mainColor),
                            color: ThemeService.mainColor,
                            tooltip: trans('Save New'),
                            onPressed: () async {
                              if (await _controller.onSaveNew())
                                setState(() => isSaved = true);
                            })
                        : Padding(
                            padding: EdgeInsets.only(right: 10),
                            child: AppButtonIconText(
                                icon: Icon(Icons.add_task,
                                    color: ThemeService.textOnMainColor),
                                color: ThemeService.secondryColor,
                                label: trans('Save New'),
                                onPressed: () async {
                                  if (await _controller.onSaveNew())
                                    setState(() => isSaved = true);
                                }))
              ],
        appBarBottom: _controller.tabs.length > 1
            ? isMobile
                ? _getTabBar()
                : Align(
                    alignment: LocalConfigService.isArabic
                        ? Alignment.centerRight
                        : Alignment.centerLeft,
                    child: Container(
                        child: _getTabBar(),
                        constraints: BoxConstraints(
                            maxWidth:
                                (_controller.tabs.length * 150).toDouble())))
            : null,
        child: Align(
            alignment: Alignment.topCenter,
            child: Container(
              constraints:
                  BoxConstraints(maxWidth: ThemeService.containerWidth),
              child: _controller.tabs.length > 1
                  ? TabBarView(
                      children: _controller.tabs.keys
                          .map<Widget>(
                              (tab) => _getTabData(_controller.tabs[tab]!))
                          .toList())
                  : _getTabData(_controller.tabs.values.first),
            )),
      ),
    );
  }

  TabBar _getTabBar() {
    return TabBar(
        labelColor: ThemeService.mainColor,
        tabs: _controller.tabs.keys
            .map((tab) => Tab(child: Text(trans(tab))))
            .toList());
  }

  Widget _getTabData(List<List<Property>> tab) {
    return SingleChildScrollView(
        child: tab.length > 1
            ? Column(
                mainAxisAlignment: MainAxisAlignment.start,
                crossAxisAlignment: CrossAxisAlignment.start,
                children: tab.map<Widget>((t) => _getData(t)).toList())
            : _getData(tab.first));
  }

  _getData(List<Property> props) {
    props.sort((a, b) => a.editOrder - b.editOrder);
    return props.length == 1 // || isMobile
        ? Container(
            width: _getWidth(props.first.size),
            margin: EdgeInsets.symmetric(horizontal: 10),
            child: PorpertyComponent(
              readOnly: _controller.readOnly,
              property: props.first,
              entity: _controller.entity,
              //filterValue: _controller.data[props.first.filterValueField],
              data: _controller.data!,
              onChange: (v) => _controller.onValueChanged(
                  v,
                  props.first
                      .dbName), //_controller.data[props.first.dbName] = v,
            ))
        : Wrap(
            alignment: WrapAlignment.start,
            children: props
                .map<Widget>((p) => Container(
                    width: _getWidth(p.size),
                    margin: EdgeInsets.symmetric(horizontal: 10),
                    child: PorpertyComponent(
                      readOnly: _controller.readOnly,
                      property: p,
                      entity: _controller.entity,
                      // onChange: (v) => _controller.data[p.dbName] = v,
                      //filterValue: _controller.data[p.filterValueField],
                      data: _controller.data!,
                      onChange: (v) => _controller.onValueChanged(v, p.dbName),
                    )))
                .toList());
  }

  double? _getWidth(PropertySizes size) {
    if (size == PropertySizes.MaxSize) return null;
    var fullwidth = LayoutService.getWidth(context) -
        MediaQuery.of(context).padding.right -
        MediaQuery.of(context).padding.left;

    if (!isMobile) fullwidth -= (ThemeService.menuWidth + 100);

    if (fullwidth > ThemeService.containerWidth) {
      fullwidth = ThemeService.containerWidth - 50;
    }

    if (size == PropertySizes.xSmall) return 100;
    if (size == PropertySizes.Small) return 150;
    //if (size == PropertySizes.Default) return 300;
    if (size == PropertySizes.Large) return 400;
    if (size == PropertySizes.xLarge) return 500;

    if (fullwidth < 400) {
      return null; // means take full width
    } else if (fullwidth < 850) {
      return (fullwidth / 2) - (isMobile ? 20 : 0);
    }
    return (fullwidth / 3) - 0;
    //return 200;
  }
}
