part of tecfy_core_client_package;

class ChatPage extends StatefulWidget {
  final User user;
  ChatPage(this.user);
  @override
  _ChatPageState createState() => _ChatPageState();
}

class _ChatPageState extends AppState<ChatPage> with TickerProviderStateMixin {
  final List<MessageRow> _messages = <MessageRow>[];
  final TextEditingController _textController = new TextEditingController();
  bool _isWriting = false;
  StreamSubscription? _subscription;
  @override
  void initState() {
    loadOld();
    _subscription = MessageService.messageStream?.listen((_message) {
      MessageRow msg = new MessageRow(
        message: _message,
        animationController: new AnimationController(
            vsync: this, duration: new Duration(milliseconds: 250)),
      );
      if (mounted)
        setState(() {
          _messages.insert(0, msg);
        });
      msg.animationController.forward();
    });
    super.initState();
  }

  loadOld() async {
    // var result = await TecfyBasicApp.apiService.httpGetDynamic('Message/GetLatest/2019-01-01T10:10:10:000');
    // print(result.data);
    List<Message>? mm = await MessageService.getLocalMessages(widget.user.id);
    if (mm == null) return;
    // await Future.delayed(Duration(seconds: 1));
    // MessageRow msg = new MessageRow(
    //   message: Message(body: 'ssss', fromUserId: 2, toUserId: 1,imSender: true,fromName: 'ss',toName: 'rr',time:DateTime.now()),
    //   animationController: new AnimationController(vsync: this, duration: new Duration(milliseconds: 250)),
    // );
    mm.sort((a, b) =>
        b.time!.millisecondsSinceEpoch - a.time!.millisecondsSinceEpoch);
    if (mounted)
      setState(() {
        _messages.addAll(mm
            .map<MessageRow>((m) => MessageRow(
                message: m,
                animationController: new AnimationController(
                    vsync: this, duration: new Duration(milliseconds: 50))))
            .toList());
      });
    _messages.forEach((m) => m.animationController.forward());

    if (!WebSocketService.connected) {
      AppNavigator.showMessage(context, 'Chat Not Connected!');
    }
  }

  @override
  Widget build(BuildContext ctx) {
    return BasePage(
      context, '${trans("Chat With")} ${widget.user.fullName}',
      // appBar: MainAppBar(
      //   context:context,
      //   title: ,
      //   actions: <Widget>[
      //     CircleAvatar(backgroundColor: Colors.white,child: Icon(Icons.person))
      //   ],
      // ),

      /// body in chat like a list in a message
      isMobile: LayoutService.isMobile(context),
      child: Column(children: <Widget>[
        new Flexible(
          child: _messages.length > 0
              ? Container(
                  child: new ListView.builder(
                    itemBuilder: (_, int index) => _messages[index],
                    itemCount: _messages.length,
                    reverse: true,
                    padding: new EdgeInsets.all(10.0),
                  ),
                )
              : NoMessage(),
        ),

        /// Line
        Divider(height: 1.5),
        Container(
          child: _buildComposer(),
          decoration: BoxDecoration(
            color: Theme.of(ctx).cardColor,
            boxShadow: [
              BoxShadow(
                blurRadius: 1.0,
                color: Colors.black12,
              )
            ],
          ),
        ),
      ]),
    );
  }

  /// Component for typing text
  Widget _buildComposer() {
    return IconTheme(
      data: IconThemeData(color: Colors.black),
      child: Container(
          margin: const EdgeInsets.symmetric(horizontal: 9.0),
          child: Row(
            children: <Widget>[
              Icon(
                Icons.add,
                color: Colors.blueAccent,
                size: 27.0,
              ),
              Flexible(
                child: Padding(
                  padding: const EdgeInsets.only(left: 5.0),
                  child: Container(
                    decoration: BoxDecoration(
                      border: Border.all(color: Colors.black12),
                    ),
                    child: Padding(
                      padding: const EdgeInsets.all(8.0),
                      child: new TextField(
                        controller: _textController,
                        maxLines: 1,
                        keyboardType: TextInputType.text,
                        textInputAction: TextInputAction.send,
                        onChanged: (String txt) {
                          if (mounted)
                            setState(() {
                              _isWriting = txt.length > 0;
                            });
                        },
                        onSubmitted: _submitMsg,
                        decoration: new InputDecoration.collapsed(
                            hintText: AppLocale.translate(
                                context, "Type a new message"),
                            hintStyle: TextStyle(color: Colors.black26)),
                      ),
                    ),
                  ),
                ),
              ),
              new Container(
                  margin: new EdgeInsets.symmetric(horizontal: 3.0),
                  child: Theme.of(context).platform == TargetPlatform.iOS
                      ? new CupertinoButton(
                          child:
                              new Text(AppLocale.translate(context, "Submit")),
                          onPressed: _isWriting
                              ? () => _submitMsg(_textController.text)
                              : null)
                      : new IconButton(
                          icon: new Icon(Icons.send),
                          onPressed: _isWriting
                              ? () => _submitMsg(_textController.text)
                              : null,
                        )),
            ],
          ),
          decoration: Theme.of(context).platform == TargetPlatform.iOS
              ? new BoxDecoration(
                  border: new Border(top: new BorderSide(color: Colors.brown)))
              : null),
    );
  }

  void _submitMsg(String txt) {
    _textController.clear();
    if (mounted) {
      setState(() {
        _isWriting = false;
      });
    }
    var message = Message.sendNew(body: txt, toUserId: widget.user.id);
    MessageService.sendMessage(message);
    MessageRow msg = new MessageRow(
      message: message,
      animationController: new AnimationController(
          vsync: this, duration: new Duration(milliseconds: 300)),
    );
    if (mounted)
      setState(() {
        _messages.insert(0, msg);
      });
    msg.animationController.forward();
  }

  @override
  void dispose() {
    _subscription?.cancel();
    for (MessageRow msg in _messages) {
      msg.animationController.dispose();
    }
    super.dispose();
  }
}

class MessageRow extends StatelessWidget {
  final Message message;
  final AnimationController animationController;

  MessageRow({required this.message, required this.animationController});

  @override
  Widget build(BuildContext ctx) {
    return new SizeTransition(
      sizeFactor: new CurvedAnimation(
          parent: animationController, curve: Curves.fastOutSlowIn),
      axisAlignment: 0.0,
      child: new Container(
        margin: const EdgeInsets.symmetric(vertical: 8.0),
        child: new Row(
          crossAxisAlignment: CrossAxisAlignment.end,
          children: <Widget>[
            new Expanded(
              child: Padding(
                padding: EdgeInsets.only(
                    left: (message.imSender ? 50 : 0),
                    right: (message.imSender ? 0 : 50)),
                child: new Column(
                  crossAxisAlignment: message.imSender
                      ? CrossAxisAlignment.end
                      : CrossAxisAlignment.start,
                  children: <Widget>[
                    Container(
                        decoration: BoxDecoration(
                          borderRadius: message.imSender
                              ? BorderRadius.only(
                                  bottomRight: Radius.circular(1.0),
                                  bottomLeft: Radius.circular(20.0),
                                  topRight: Radius.circular(20.0),
                                  topLeft: Radius.circular(20.0))
                              : BorderRadius.only(
                                  bottomRight: Radius.circular(20.0),
                                  bottomLeft: Radius.circular(1.0),
                                  topRight: Radius.circular(20.0),
                                  topLeft: Radius.circular(20.0)),
                          color: message.imSender
                              ? Color(0xFF6991C7).withOpacity(0.6)
                              : Colors.red[100],
                        ),
                        padding: const EdgeInsets.all(10.0),
                        child: new Text(
                          message.body,
                        ) //style: Style.colorMessage,),
                        ),
                    Text(
                      message.timeString,
                      style: TextStyle(fontSize: 9),
                      textDirection: TextDirection.ltr,
                    )
                  ],
                ),
              ),
            ),
          ],
        ),
      ),
    );
  }
}

class NoMessage extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return Center(
        child: Container(
            child: Text(AppLocale.translate(
                context, "Not Have Message")) //,style: Style.notHaveMessage,),
            ));
  }
}

//   @override
//   Widget build(BuildContext context) {

//     return Scaffold(
//         appBar: MainAppBar(context: context,title: "Chat"),
//         body: Column(
//           children: <Widget>[
//             Expanded(
//               child: ListView(
//                 children: <Widget>[],
//               ),
//             ),
//             Row(
//               children: <Widget>[
//                 IconButton(icon: Icon(Icons.attach_file),),
//                 Expanded(
//                   child: TextFormField(),
//                 ),
//                 IconButton(icon: Icon(Icons.send),),
//               ],
//             )
//           ],
//         ),
//         );
//   }

// }
